@@ -14,59 +14,20 @@ from tkinter.messagebox import showerror
 from tkinter.ttk import Separator, Scale, Combobox, Checkbutton, Spinbox, Style
 from typing import Optional, Mapping
 
+from .level_meter import BarLevelMeter, DotLevelMeter
+from .db_calculator import DbCalculator
 from ..common.audio_devices import get_device_list, get_device_index
 from ..common.config import default_config, config_path
-from ..common.constants import SAMPLING_RATE, FRAME_SIZE, VOCAL_GROUPS, SATURATION_LEVEL
+from ..common.constants import SAMPLING_RATE, FRAME_SIZE, VOCAL_GROUPS
 from ..common.log import logger
 from ..common.singleton import register_process, unregister_process
-from ..common.utils import my_assert, to_gain, remove_non_ascii
+from ..common.utils import my_assert, remove_non_ascii
 from .calibration_tool import generate_test_audio, MAX_LATENCY, calculate_latency
 from .gui_enums import GuiEvent, GuiState
-from .signal_level import SignalLevel
 from .signal_smoother import SignalSmoother
 from .slave_client import SlaveClient
 
 
-class LevelMeter:
-    class LED:
-        def __init__(self, canvas, idx: int):
-            self.canvas = canvas
-            self.tag = canvas.create_oval(10 + 20 * idx, 10, 20 + 20 * idx, 20, fill='white')
-            # 0   1   2   3   4   5   6   7   8   9
-            # -18 -16 -14 -12 -10 -8 -6  -4  -2  0 dB
-            if idx < 5:
-                self.color = 'grey'
-            elif idx < 8:
-                self.color = 'green'
-            elif idx < 9:
-                self.color = 'orange'
-            else:
-                self.color = 'red'
-
-        def on(self):
-            self.canvas.itemconfigure(self.tag, fill=self.color)
-
-        def off(self):
-            self.canvas.itemconfigure(self.tag, fill='white')
-
-    def __init__(self, master, **kwargs):
-        canvas = tk.Canvas(master=master, width=210, height=20)
-        self.dots = [self.LED(canvas, i) for i in range(10)]
-        self.prev_level = 0
-        canvas.grid(**kwargs)
-
-    def update_level(self, level: int):
-        if not 0 <= level <= 10:
-            raise ValueError(f"Invalid level {level}")
-        if level > self.prev_level:
-            for dot in range(self.prev_level, level):
-                self.dots[dot].on()
-        if level < self.prev_level:
-            for dot in range(level, self.prev_level):
-                self.dots[dot].off()
-        self.prev_level = level
-
-
 # TODO:
 # * test dead stream
 # * handle bad config in sessions
@@ -110,14 +71,14 @@ class AudioSettings:
         self.input_selector.grid(row=row, column=1, sticky='EW', padx=10)
         row += 1
         tk.Label(self.top, text="Input level:").grid(row=row, column=0, sticky='W')
-        self.monitor = LevelMeter(self.top, row=row, column=1)
+        self.monitor = BarLevelMeter(self.top, row=row, column=1)
         row += 1
         tk.Label(self.top, text="Microphone Gain:").grid(row=row, column=0, sticky='W')
-        gain = float(config.get('gain', 0))
-        level_scale = Scale(self.top, from_=-10, to=10, command=self._set_gain)
+        gain = config.get('gain', 0)
+        self._db_calculator = DbCalculator(gain_db=gain)
+        level_scale = Scale(self.top, from_=-20, to=20, command=self._set_gain)
         level_scale.set(gain)
         level_scale.grid(row=row, column=1, sticky='EW', padx=10)
-        self.gain = to_gain(gain)
         row += 1
 
         # Output
@@ -280,7 +241,8 @@ class AudioSettings:
         my_assert(self.rec is not None, "This thread should not start if rec is None")
         if self.rec:
             if self.cursor == -1:  # recording complete
-                latency = calculate_latency(self.out_signal[:self.weep_len].mean(axis=1), self.in_signal * self.gain)
+                latency = calculate_latency(self.out_signal[:self.weep_len].mean(axis=1),
+                                            self.in_signal * self._db_calculator.gain)
                 logger.info(f"Processing done. Latency is {latency or 'not measured successfully'}")
                 self._set_latency(int(round(latency or 0)))
             else:  # recording incomplete
@@ -298,27 +260,20 @@ class AudioSettings:
 
     def _level_updater(self):
         logger.debug("Start level updater")
-        smoother = None
+        stream_active = False
         while True:
             sample = self.level_changes.get()
             if sample is None:  # poison pill
                 break
             if isinstance(sample, int):  # stream ended
-                smoother = None
-                self.monitor.update_level(0)
+                self.monitor.reset()
+                stream_active = False
             else:  # actual numpy array
-                if smoother is None:
-                    smoother = SignalSmoother(0)
+                if not stream_active:
                     self._stream_active()
-                peak = np.abs(sample).max()
-                if peak * self.gain >= SATURATION_LEVEL:
-                    level = 10
-                else:
-                    dbs = SignalLevel.calculate_dbs(sample * self.gain)
-                    level = max(1, min(9, int(round(dbs / 2 + 10))))
-                level = smoother(level)
-                if level is not None:
-                    self.monitor.update_level(level)
+                    stream_active = True
+                db = self._db_calculator.from_sample(sample)
+                self.monitor.update_level(db)
         logger.debug("End level updater")
 
     def _stream_active(self):
@@ -327,9 +282,10 @@ class AudioSettings:
         self.config['audio_input'] = remove_non_ascii(self.input_selector.get())
         self.config['audio_output'] = remove_non_ascii(self.output_selector.get())
 
-    def _set_gain(self, gain):
-        self.config['gain'] = str(gain)
-        self.gain = to_gain(gain)
+    def _set_gain(self, gain: str):  # gain is in dB and is a string because it is a tkinter widget
+        self.config['gain'] = gain
+        self._db_calculator.set_gain(gain_db=gain)
+        logger.debug(f"Gain set to {self._db_calculator.gain} ({self._db_calculator.gain_db} dB)")
 
     def test_output(self):
         logger.info("Testing output")
@@ -414,9 +370,7 @@ class SlaveClientGui:
                                 command=partial(self.send_event, GuiEvent.SETTINGS_BUTTON.value))
         self.button.grid(row=3, column=1, columnspan=2, padx=10, pady=10)
 
-        self.monitor = tk.Canvas(master=self.root, width=20, height=20)
-        self.monitor.grid(row=1, column=2, padx=10, pady=10)
-        self.monitor_tag = self.monitor.create_oval(10, 10, 20, 20, fill='white')
+        self.monitor = DotLevelMeter(self.root, row=1, column=2, padx=10, pady=10)
 
         # State variables
         self.state = None
@@ -455,17 +409,10 @@ class SlaveClientGui:
         self.loop.call_soon_threadsafe(self.events_out.put_nowait, event)
 
     # this is to be called from another thread
-    def set_signal_level(self, level: SignalLevel):
+    def set_signal_level(self, db: Optional[float]):
         if self.state == GuiState.DEAD:
             return  # ignore post-mortem commands
-        if level == SignalLevel.white:  # no delay in white
-            self.monitor.itemconfigure(self.monitor_tag, fill=level.name)
-            self.signal_smoother.reset()
-            return
-        level = self.signal_smoother(level)
-        if level is not None:
-            logger.debug(f"Changing level to {level.name}")
-            self.monitor.itemconfigure(self.monitor_tag, fill=level.name)
+        self.monitor.update_level(db)
 
     def open_settings(self, config: Mapping[str, str]):
         self.settings = AudioSettings(self.root, self.close_settings, config)
